package com.rf.kjb.scale.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.rf.kjb.scale.dao.model.GameRecordEntity;
import com.rf.kjb.scale.dao.model.RecordPhyEntry;
import com.rf.kjb.scale.dao.repository.GameRecordRepository;
import com.rf.kjb.scale.service.GameRecordService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Author:zzf
 * @Date:2023/6/7:10:26
 * @Description:
 */
@Service
public class GameRecordServiceImpl implements GameRecordService {
    @Autowired
    private GameRecordRepository gameRecordRepository;
    @Override
    public void save(GameRecordEntity gameRecordEntity) {
        this.gameRecordRepository.save(gameRecordEntity);
    }

    @Override
    public Page<GameRecordEntity> find(String json) {
        JSONObject jsonObject = JSONObject.parseObject(json);
        int pageNum = 1;
        int pageSize = 10 ;

        if(jsonObject.containsKey("pageNum")&&jsonObject.getIntValue("pageNum")>0){
            pageNum = jsonObject.getIntValue("pageNum");
        }
        if(jsonObject.containsKey("pageSize")&&jsonObject.getIntValue("pageSize")>0){
            pageSize = jsonObject.getIntValue("pageSize");
        }
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Specification<RecordPhyEntry> specification = (root, query, cb) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if(jsonObject.containsKey("id")&& StringUtils.isNotBlank(jsonObject.getString("id"))){
                predicateList.add(cb.equal(root.get("id"),jsonObject.getString("id")));
            }
            if(jsonObject.containsKey("num1") && StringUtils.isNotBlank(jsonObject.getString("num1"))){
                predicateList.add(cb.like(root.get("num1"),"%"+jsonObject.getString("num1")+"%"));
            }
            if(jsonObject.containsKey("gameScene") && StringUtils.isNotBlank(jsonObject.getString("gameScene"))){
                predicateList.add(cb.equal(root.get("gameScene"),jsonObject.getString("gameScene")));
            }
            if(jsonObject.containsKey("equipmentType") && StringUtils.isNotBlank(jsonObject.getString("equipmentType"))){
                predicateList.add(cb.equal(root.get("equipmentType"),jsonObject.getString("equipmentType")));
            }
            if(jsonObject.containsKey("gameDifficulty") && StringUtils.isNotBlank(jsonObject.getString("gameDifficulty"))){
                predicateList.add(cb.like(root.get("gameDifficulty"),jsonObject.getString("gameDifficulty")));
            }
            if(jsonObject.containsKey("type") && StringUtils.isNotBlank(jsonObject.getString("type"))){
                predicateList.add(cb.equal(root.get("type"),jsonObject.getString("type")));
            }
            if(jsonObject.containsKey("gameTime") && StringUtils.isNotBlank(jsonObject.getString("gameTime"))){
                predicateList.add(cb.equal(root.get("gameTime"),jsonObject.getString("gameTime")));
            }
            if(jsonObject.containsKey("startDate") && StringUtils.isNotBlank(jsonObject.getString("startDate"))){
                try {
                    predicateList.add(cb.greaterThanOrEqualTo(root.get("createTime"),simpleDateFormat.parse(jsonObject.getString("startDate")+" 00:00:00")));
                } catch (ParseException e) {
                    throw new RuntimeException(e);
                }
            }
            if(jsonObject.containsKey("endDate") && StringUtils.isNotBlank(jsonObject.getString("endDate"))){
                try {
                    predicateList.add(cb.lessThanOrEqualTo(root.get("createTime"),simpleDateFormat.parse(jsonObject.getString("endDate")+" 23:59:59")));
                } catch (ParseException e) {
                    throw new RuntimeException(e);
                }
            }
            query.where(cb.and(predicateList.toArray(new Predicate[0])));
            query.orderBy(cb.desc(root.get("saveTime")));
            return query.getRestriction();
        };
        return this.gameRecordRepository.findAll(specification, PageRequest.of(pageNum-1,pageSize));
    }

    @Override
    public GameRecordEntity findById(String id) {
        return this.gameRecordRepository.findById(id).get();
    }
}
